<?php
### Lynx Template
### Copyright (c) 2011 Jonathan Coulet - j.coulet@gmail.com
### http://lynx-template.phpixel.fr/
### Licensed under the GPL license.
###
### Lynx Template est basé sur le moteur de template Simple TPL (Copyright (c) 2009 Jonathan Coulet - j.coulet@gmail.com)
###
### TODO
### - helpers personalisés (fonctions utilisateurs dans dossier helpers/)
### - ELSELOOP => (contenu alternatif si une boucle est vide)
### - append() prepend() template (similaire INC) ??
### - loopToVar('loopName', 'varName') => remplace la variable par le contenu d'un LOOP parssé ??
###
### V 1.2
### - réécriture de la méthode parse()
### - ajout du helper nl2br
### - il est maintenant possible d'appliquer plusieurs helpers à une variable
### - les blocs répétitifs non parssés sont maintenant supprimés
### - ajout d'un paramètre optionnel ($prefix) à la méthode assignVars()
### - suppression des variables non parssées
### - correction helper truncat
### - ajout méthode reverseLoop()
###
### V 1.1
### - réécriture des méthode loop(), assignVar(), remove(), parseVar() et assignVars()
### - optimisation de la méthode parse
### - ajout méthode copyLoop()
### - ajout du helper magique
### - ajout du temps d'execution de la méthode parse() dans l'affichage de debug()
###
### V 1.0
### - suppressions méthodes compression(), retire() et de la propriété $param
### - ajout des propriétés $buffer et $dir
### - renommages des propriétés, méthodes et helpers variables
### - modification de la syntaxe template
### - ajout méthodes debug(), display(), removeBlock()

class lynxTemplate{
	private $dir;
	private $buffer;
	private $aVar;
	private $aLoop;
	private $aOpt;
	private $parseTime;
	
	public function __construct($file, $dir = 'templates/'){
		$this->buffer = file_get_contents($dir.$file);
		$this->aVar = array();
		$this->aLoop = array();
		$this->aOpt = array();
		$this->dir = $dir;
	}
	
	###
	### assignVars()
	### alimente le tableau $aVar
	###
	
	public function assignVars($vars, $prefix = ''){
		foreach($vars as $name=>$value){
			if($prefix != '') $name = $prefix.'_'.$name;
			$this->assignVar($name, $value);
		}
	}
	
	###
	### assignVar()
	### alimente le tableau $aVar
	###
	
	public function assignVar($name, $value){
		$this->aVar[$name] = $value;
	}
	
	###
	### loop()
	### alimente le tableau $aLoop
	###
	
	public function loop($name, $vars){
		if(!isset($this->aLoop[$name])) $this->aLoop[$name] = array();
		array_push($this->aLoop[$name], $vars);
	}
	
	###
	### remove()
	### alimente le tableau $aOpt
	###
	
	public function remove($name){
		$this->aOpt[] = $name;
	}
	
	###
	### copyLoop()
	### déclare un nouveau bloc répétitif et lui assigne les valeurs d'un existant
	###
	
	public function copyLoop($source, $target){
		$this->aLoop[$target] = $this->aLoop[$source];
	}
	
	###
	### reverseLoop()
	###
	
	public function reverseLoop($name){
		$this->aLoop[$name] = array_reverse($this->aLoop[$name]);
	}
	
	### 
	### debug()
	###
	
	public function debug(){
		echo '<b>parseTime : </b>'.$this->parseTime.'<br />';
		echo '<b>$aVar :</b><br />';
		print_r($this->aVar);
		echo '<br />';
		echo '<b>$aLoop :</b><br />';
		print_r($this->aLoop);
		echo '<br />';
		echo '<b>$aOpt :</b><br />';
		print_r($this->aOpt);
		echo '<br />';
	}
	
	### 
	### display()
	###
	
	public function display(){
		echo $this->parse();
	}
	
	### 
	### parse()
	###
	
	public function parse(){
		// start time
		$timeStart = microtime(true);
		// Import des includes dans le buffer
		if(preg_match_all('#<!--INC:([a-z-0-9-\._-]+)-->#Usi', $this->buffer, $matches)){
			foreach($matches[1] as $k=>$file){
				if(file_exists($this->dir.$file)){
					$content = file_get_contents($this->dir.$file);
					$this->buffer = str_replace($matches[0][$k], $content, $this->buffer);
				}
			}
		}
		// Suppression des blocs optionnels
		foreach($this->aOpt as $name){
			$this->removeBlock('opt', $name);
		}
		// Parssage des variables
		foreach($this->aVar as $name=>$value){
			$this->buffer = $this->parseVar($name, $value, $this->buffer);
		}
		// Parssage des blocs répétitifs
		foreach($this->aLoop as $loopName=>$loopVars){
			$output = '';
			// Récupération de la ligne
			preg_match('#(<!--LOOP:'.$loopName.'-->)(.*)(<!--ENDLOOP:'.$loopName.'-->)#Usi', $this->buffer, $matches);
			$input = $matches[2];
			// Boucle et parssage des variables
			foreach($loopVars as $vars){
				$temp = $input;
				foreach($vars as $name=>$value){
					$temp = $this->parseVar($loopName.'.'.$name, $value, $temp);
				}
				$output .= $temp;
			}
			// Parssage final
			$this->buffer = str_replace('<!--LOOP:'.$loopName.'-->'.$input.'<!--ENDLOOP:'.$loopName.'-->', $output, $this->buffer);
		}
		// Supression des blocs répétitifs non parssés
		$this->buffer = preg_replace('#<!--LOOP:(.*)-->(.*)<!--ENDLOOP:(.*)-->#Usi', '', $this->buffer);
		// suppression des variables non parssées
		$this->buffer = preg_replace('#{([a-z-0-9-:,_]+)}#Usi', '', $this->buffer);
		// end time
		$this->parseTime = microtime(true)-$timeStart;
		// end
		return $this->buffer;
	}
	
	###
	### removeBlock()
	### supprime un bloc et son contenu
	### si $removeContent est false le contenu ne sera pas supprimé
	###
	
	private function removeBlock($type, $name, $removeContent = true){
		if($removeContent) $mask = '#<!--'.mb_strtoupper($type).':'.$name.'-->(.*)<!--END'.mb_strtoupper($type).':'.$name.'-->#Usi';
		else $mask = '#<!--(END'.mb_strtoupper($type).'|'.mb_strtoupper($type).'):'.$name.'-->#Usi';
		$this->buffer = preg_replace($mask, '', $this->buffer);
	}
	
	###
	### parseVar()
	### parse les variables d'une chaine (applique les helpers)
	###
	
	private function parseVar($name, $value, $content){
		$tag = '{'.$name.'}';
		if(preg_match('#{'.$name.':([a-z-0-9-,_]+)}#Usi', $content, $matches)){
			$tag = $matches[0];
			foreach(explode(',', $matches[1]) as $helper){
				switch($helper){
					case 'ucw':
						$value = ucwords($value);
						break;
					case 'up':
						$value = mb_strtoupper($value);
						break;
					case 'low':
						$value = mb_strtolower($value);
						break;
					case 'ucf':
						$value = ucfirst($value);
						break;
					case 'nl2br':
						$value = nl2br($value);
					default:
						if(is_numeric($helper) && mb_strlen($value) > $helper) $value = mb_strcut($value, 0, $helper).'...';
				}	
			}
		}
		return str_replace($tag, $value, $content);
	}
}
?>